@@ -1,3 +1,9 @@
+"""
+Type-safe data interchange for Python data classes.
+
+:see: https://github.com/hunyadi/strong_typing
+"""
+
 import dataclasses
 import datetime
 import enum
@@ -67,7 +73,10 @@ def is_named_tuple_type(typ: type) -> TypeGuard[Type[NamedTuple]]:
 
     typ = unwrap_annotated_type(typ)
 
-    b = typ.__bases__
+    b = getattr(typ, "__bases__", None)
+    if b is None:
+        return False
+
     if len(b) != 1 or b[0] != tuple:
         return False
 
@@ -189,6 +198,21 @@ def is_type_literal(typ: type) -> bool:
     return typing.get_origin(typ) is Literal
 
 
+def unwrap_literal_value(typ: type) -> Any:
+    """
+    Extracts the single constant value captured by a literal type.
+
+    :param typ: The literal type `Literal[value]`.
+    :returns: The values captured by the literal type.
+    """
+
+    args = unwrap_literal_values(typ)
+    if len(args) != 1:
+        raise TypeError("too many values in literal type")
+
+    return args[0]
+
+
 def unwrap_literal_values(typ: type) -> Tuple[Any, ...]:
     """
     Extracts the constant values captured by a literal type.
@@ -353,6 +377,15 @@ def get_class_properties(typ: type) -> Iterable[Tuple[str, type]]:
         return resolved_hints.items()
 
 
+def get_class_property(typ: type, name: str) -> Optional[type]:
+    "Looks up the annotated type of a property in a class by its property name."
+
+    for property_name, property_type in get_class_properties(typ):
+        if name == property_name:
+            return property_type
+    return None
+
+
 def get_referenced_types(typ: type) -> List[type]:
     """
     Extracts types indirectly referenced by this type.
@@ -589,9 +622,9 @@ def check_recursive(
     obj: Any,
     /,
     *,
-    pred: Callable[[Type[T], T], bool] = None,
-    type_pred: Callable[[Type[T]], bool] = None,
-    value_pred: Callable[[T], bool] = None,
+    pred: Optional[Callable[[Type[T], T], bool]] = None,
+    type_pred: Optional[Callable[[Type[T]], bool]] = None,
+    value_pred: Optional[Callable[[T], bool]] = None,
 ) -> bool:
     """
     Checks if a predicate applies to all nested member properties of an object recursively.
