@@ -1,10 +1,19 @@
+"""
+Type-safe data interchange for Python data classes.
+
+:see: https://github.com/hunyadi/strong_typing
+"""
+
 import keyword
+from typing import Optional
 
 from .auxiliary import Alias
 from .inspection import get_annotation
 
 
-def python_field_to_json_property(python_id: str, python_type: type = None) -> str:
+def python_field_to_json_property(
+    python_id: str, python_type: Optional[type] = None
+) -> str:
     """
     Map a Python field identifier to a JSON property name.
 
